package require Tk

set scriptdir [file dirname [info script]]
set filterfile [file join $scriptdir filters.tcl]


proc Init {root ns} {
  variable liste 
  variable filtercodes
  variable filterfile
  variable codewidget
  variable filtercodes
  variable saved_filters
  variable popupmenu
  # root is the widget pathname for a ttk::frame
  # where one should place the GUI
  
  # grrr Aqua right button handler
  if {[tk windowingsystem]=="aqua"} {
    set rightclick <2>
  } else {
    set rightclick <3>
  }  

  # create listbox with available filters 
  ttk::label $root.l_filtername -text "Name" -anchor e
  ttk::label $root.l_filterselect -text "Available filters" -anchor center
  ttk::entry $root.filtername -textvariable ${ns}::filtername
  scrollit $root.f_filterselect {
    set liste [listbox $root.f_filterselect.filterselect -listvariable ${ns}::saved_filters]
  }  
  bind $liste <<ListboxSelect>> ${ns}::selected
  bind $liste <Double-1> [list ${ns}::doubleclick %x %y]
  bind $liste $rightclick [list ${ns}::rightclick %x %y %X %Y]
  set bfr_ls [ttk::frame $root.loadsave]
  ttk::button $bfr_ls.save -text "Save" -command ${ns}::savecode
  ttk::button $bfr_ls.load -text "Load" -command ${ns}::loadcode
  pack $bfr_ls.save $bfr_ls.load -side left 


  set popupmenu [menu $root.popup]
  $popupmenu add command -label "Load" -command ${ns}::loadcode
  $popupmenu add command -label "Delete" -command ${ns}::delete

  # Text widget where to type the code
  ttk::label $root.l_code -text {Code (input image is $input):}
  scrollit $root.f_code {
    set codewidget [text $root.f_code.code -wrap none]
  }
  $codewidget mark set limit 1.0
  # Tab-key autocompletion
  # a dirty little trick:
  # we use tkcon::Expand, which does all the magic 
  # on any text widget. The limit mark must be set at the beginning
  bind $codewidget <Key-Tab> [list ${ns}::autocomplete %W]

  # Buttons for compiling and running filtercode
  #
  set bfr [ttk::frame $root.bfr]
  ttk::button $bfr.about -text  "Help" -command ${ns}::popup
  ttk::button $bfr.compile -text "Compile" -command ${ns}::compile
  ttk::button $bfr.run -text "Run" -command ${ns}::run
  ttk::button $bfr.clear -text "Clear" -command ${ns}::clear
  ttk::checkbutton $bfr.framedrop -text "Frame dropping" -variable ::XRlab::enableframedrop
  pack $bfr.about $bfr.compile $bfr.run $bfr.clear $bfr.framedrop -side left 


  # do the layout
  grid $root.l_filtername $root.filtername -sticky nsew
  grid $root.l_code $root.l_filterselect
  grid $root.f_code   $root.f_filterselect -sticky nsew
  grid $bfr $bfr_ls
  
  grid rowconfigure $root 1 -weight 1
  grid columnconfigure $root 0 -weight 1
 
  # example filtercodes
  set filtercodes [dict create \
       Gruß {puts "Servus!"
}      Gaindiv {set qrb [XRlab::qrbadpix $input]
$qrb /= schuessel
return $qrb
}      Gaindiv_init {bild16 sch16 Gainbild.his
bildfloat schuessel [XRlab::qrbadpix sch16]
return schuessel
}]
  # read the stored filter codes - overwrite examples
  # if existent
  if {![catch {set fd [open $filterfile r]}]} {
    fconfigure $fd -encoding utf-8 -translation lf
    set filtercodes [dict create {*}[read $fd]]
    close $fd
  }
  set saved_filters [lsort [dict keys $filtercodes]]

  # install the filterproc
  ::XRlab::installfilter "Autofilter" ::autofilter_body
  return "Auto Filter"
}

# helper functions to attach vertical&horizontal scrollbars
proc scrollit {fpath wcmd} {
  ttk::frame $fpath
  set wpath [uplevel $wcmd]
  
  scrollbar $fpath.xscroll -orient horizontal -command [list $wpath xview]
  scrollbar $fpath.yscroll -orient vertical -command   [list $wpath yview]
  $wpath configure -xscrollcommand [list $fpath.xscroll set] -yscrollcommand [list $fpath.yscroll set]
  grid $wpath $fpath.yscroll -sticky nsew
  grid $fpath.xscroll x -sticky nsew

  grid columnconfigure $fpath 0 -weight 1
  grid rowconfigure $fpath 0 -weight 1

  return $fpath
}

proc ::autofilter_body {input} {
  return $input
}

proc compile {} {
  variable codewidget
  set code [$codewidget get 1.0 end]
  proc ::autofilter_body {input} "$code\nreturn \$input"
}

proc run {} {
  compile
  uplevel ::autofilter_body $::XRlab::imgname
}

proc savetofilterfile {} {
  # write the filters to the filterfile
  variable filterfile
  variable filtercodes
  set fd [open $filterfile w]
  fconfigure $fd -encoding utf-8 -translation lf
  puts $fd $filtercodes
  close $fd
} 


proc savecode {} {
  variable saved_filters
  variable filtercodes
  variable codewidget
  variable filtername
  variable filterfile
  set code [$codewidget get 1.0 end]
  dict set filtercodes $filtername $code
  set saved_filters [lsort [dict keys $filtercodes]]

  savetofilterfile
  $codewidget edit modified false
}

proc loadcode {} {
  variable filtercodes
  variable codewidget
  variable filtername
  if {[$codewidget edit modified]} {
    set ans [tk_messageBox -icon warning -type yesno -message "The current filter code is modified." -detail "Do you want me to load $filtername and discard any changes?" -title "Modified Filtercode" -default no]
    if {!$ans} {return false}
  }  
  if {![catch {dict get $filtercodes $filtername} code]} {
     $codewidget delete 1.0 end
     $codewidget insert end $code
     $codewidget mark set limit 1.0
     $codewidget edit modified false
  }   
  compile
}

proc clear {} {
  variable codewidget
  $codewidget delete 1.0 end
}


proc selected {} {
  variable liste
  variable filtername
  set selected [$liste curselection]
  if {[llength $selected]==1} {
    set filtername [$liste get [lindex $selected 0]]
  }  
}

proc doubleclick {x y} {
  loadcode
}

proc rightclick {x y X Y} {
  variable liste
  variable popupmenu
  # first, do the same as when pressing button 1
  $liste selection clear 0 end
  $liste selection set @$x,$y
  $liste activate @$x,$y
  event generate $liste <<ListboxSelect>>
  $popupmenu post $X $Y
}

proc delete {} {
  variable filtername
  variable filtercodes
  dict unset filtercodes $filtername
  variable saved_filters [lsort [dict keys $filtercodes]]
  savetofilterfile
} 


proc autocomplete {w} {
  # set limit for completion at the beginning of the 
  # current line
  $w mark set limit "insert linestart"
  ::tkcon::Expand $w
  return -code break
}


# 
proc popup {} {
 variable filterfile
 tk_messageBox -type ok -message "\
 Autofilter plugin 1.0
 (C) 2009 Auriocus

 The easy way to write a filter function. Type code into\
 the text field. The input image (read from .his or .pgm)\
 is stored in a bild16 \$input. Return the name of an object\
 (bildfloat, bild16 or bildcomplex) which should be shown.
 Commands:
 Compile activates the current code as a filter
 Run runs the current code once
 Save adds the current code as a new filter in the list
 Load (also double-click on a list item) loads the selected code\
 from the filterlist
 Tab-key auto-completes the code in the code editor 
 Rightclick on a list item opens a popup menu 
 The filters are saved in $filterfile"
}


